@@ -36,6 +36,16 @@
 ! if(FlagIsSet(F_FED_PARROT, F_TICKET_OK, F_SAVED_CAT)) returns true if all three flags are set
 ! if(FlagIsClear(F_TICKET_OK, F_SAVED_CAT)) returns true if both flags are clear
 
+! There are also functions to check if any of two or three flags are set or clear:
+! if(AnyFlagIsSet(F_FED_PARROT, F_TICKET_OK, F_SAVED_CAT)) returns true if any of the three flags are set
+! if(AnyFlagIsClear(F_FED_PARROT, F_TICKET_OK, F_SAVED_CAT)) returns true if any of the three flags are clear
+
+! In all of these procedures, you can use a negative flag number, to mean "do
+! the opposite." E.g. SetFlag(F_FED_PARROT, -F_SAVED_CAT) will set 
+! F_FED_PARROT and clear F_SAVED_CAT, while 
+! AnyFlagIsSet(F_FED_PARROT, -F_TICKET_OK) returns true if F_FED_PARROT is set
+! or F_TICKET_OK is clear.
+
 System_file;
 
 Constant EXT_FLAGS = 1;
@@ -47,7 +57,9 @@ Constant RUNTIME_ERRORS = 2;
 Constant RTE_MINIMUM = 0;
 Constant RTE_NORMAL = 1;
 Constant RTE_VERBOSE = 2;
-Constant TM_NOT_PUNY;
+#Endif;
+#Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
+Constant FL_ERR = "^[Flags error #";
 #Endif;
 
 #Ifv3;
@@ -66,21 +78,22 @@ Array game_flags -> (FLAG_COUNT + 1) / 8 + ((FLAG_COUNT + 1) & 7 > 0);
 
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 [ IncorrectFlagNumber p_x;
+	if(p_x < 0) p_x = -p_x;
 #Ifdef DEBUG;
 		if(p_x == 0) {
 	#Iftrue RUNTIME_ERRORS == RTE_VERBOSE;
-			"WARNING: Flags: Use of flag 0 is deprecated, and will not work as second or third argument to flag functions.";
+			print (string) FL_ERR,"2: Warning: Use of flag 0 is deprecated, and will not work as second or third argument to flag functions]^";
 	#Ifnot;
-			"WARNING: Flags #2!";
+			print (string) FL_ERR,"2 (warning)]^";
 	#Endif;
 		}
 #Endif;
-	if(p_x < 0 || p_x > FLAG_COUNT) {
+	if(p_x > FLAG_COUNT) {
 #Iftrue RUNTIME_ERRORS == RTE_VERBOSE;
-		"ERROR: Flags: Tried to use flag ", p_x, 
-			", but the highest flag number is ", FLAG_COUNT, "!";
+		print_ret (string) FL_ERR,"1: Tried to use flag ", p_x,
+			", but the highest flag number is ", FLAG_COUNT, "]";
 #Ifnot;
-		"ERROR: Flags #1!";
+		print_ret (string) FL_ERR,"1]";
 #Endif;
 	}
 	rfalse;
@@ -91,6 +104,11 @@ Array game_flags -> (FLAG_COUNT + 1) / 8 + ((FLAG_COUNT + 1) & 7 > 0);
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 	if(IncorrectFlagNumber(p_x)) rfalse;
 #Endif;
+	if(p_y) SetFlag(p_y, p_z);
+	if(p_x < 0) {
+		ClearFlag(-p_x);
+		rtrue;
+	}
 #IfV5;
 	@log_shift p_x (-3) -> _val;
 	p_x = p_x & 7;
@@ -100,13 +118,17 @@ Array game_flags -> (FLAG_COUNT + 1) / 8 + ((FLAG_COUNT + 1) & 7 > 0);
 	_val = p_x / 8;
 	game_flags -> _val = game_flags -> _val | flag_powers -> (p_x & 7);
 #EndIf;
-	if(p_y) SetFlag(p_y, p_z);
 ];
 
 [ ClearFlag p_x p_y p_z _val;
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 	if(IncorrectFlagNumber(p_x)) rfalse;
 #Endif;
+	if(p_y) ClearFlag(p_y, p_z);
+	if(p_x < 0) {
+		SetFlag(-p_x);
+		rtrue;
+	}
 #IfV5;
 	@log_shift p_x (-3) -> _val;
 	p_x = p_x & 7;
@@ -116,41 +138,57 @@ Array game_flags -> (FLAG_COUNT + 1) / 8 + ((FLAG_COUNT + 1) & 7 > 0);
 	_val = p_x / 8;
 	game_flags -> _val = game_flags -> _val & ~ flag_powers -> (p_x & 7);
 #EndIf;
-	if(p_y) ClearFlag(p_y, p_z);
 ];
 
-[ FlagIsSet p_x p_y p_z _val;
+[ _FlagValue p_x _abs_p_x _val;
+	if(p_x >= 0) _abs_p_x = p_x; else _abs_p_x = -p_x;
+#IfV5;
+	@log_shift _abs_p_x (-3) -> _val; ! Divide by 8
+	_abs_p_x = _abs_p_x & 7;
+	@log_shift 1 _abs_p_x -> _abs_p_x;
+	_val = game_flags -> _val & _abs_p_x;
+#IfNot;
+	_val = _abs_p_x / 8;
+	_val = game_flags -> _val & flag_powers -> (_abs_p_x & 7);
+#EndIf;
+	if(p_x >= 0) return _val;
+	if(_val == 0) rtrue;
+	rfalse;
+];
+
+[ FlagIsSet p_x p_y p_z;
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 	if(IncorrectFlagNumber(p_x)) rfalse;
 #Endif;
-#IfV5;
-	@log_shift p_x (-3) -> _val;
-	p_x = p_x & 7;
-	@log_shift 1 p_x -> p_x;
-	_val = game_flags -> _val & p_x;
-#IfNot;
-	_val = p_x / 8;
-	_val = game_flags -> _val & flag_powers -> (p_x & 7);
-#EndIf;
-	if(_val == 0) rfalse;
+	if(_FlagValue(p_x) == 0) rfalse;
 	if(p_y) return FlagIsSet(p_y, p_z);
 	rtrue;
 ];
 
-[ FlagIsClear p_x p_y p_z _val;
+[ AnyFlagIsSet p_x p_y p_z;
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 	if(IncorrectFlagNumber(p_x)) rfalse;
 #Endif;
-#IfV5;
-	@log_shift p_x (-3) -> _val;
-	p_x = p_x & 7;
-	@log_shift 1 p_x -> p_x;
-	_val = game_flags -> _val & p_x;
-#IfNot;
-	_val = p_x / 8;
-	_val = game_flags -> _val & flag_powers -> (p_x & 7);
-#EndIf;
-	if(_val) rfalse;
+	if(_FlagValue(p_x)) rtrue;
+	if(p_y) return AnyFlagIsSet(p_y, p_z);
+	rfalse;
+];
+
+[ FlagIsClear p_x p_y p_z;
+#Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
+	if(IncorrectFlagNumber(p_x)) rfalse;
+#Endif;
+	if(_FlagValue(p_x)) rfalse;
 	if(p_y) return FlagIsClear(p_y, p_z);
 	rtrue;
 ];
+
+[ AnyFlagIsClear p_x p_y p_z;
+#Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
+	if(IncorrectFlagNumber(p_x)) rfalse;
+#Endif;
+	if(_FlagValue(p_x) == 0) rtrue;
+	if(p_y) return AnyFlagIsClear(p_y, p_z);
+	rfalse;
+];
+
