@@ -2,8 +2,9 @@
 !
 System_file;
 
-Constant PUNYINFORM_MAJOR_VERSION = 4;
-Constant PUNYINFORM_MINOR_VERSION = 4;
+Constant PUNYINFORM_MAJOR_VERSION = 5;
+Constant PUNYINFORM_MINOR_VERSION = 14;
+Constant PUNYINFORM_PATCH_VERSION = 1; ! Usually 0 (if zero, it is not printed in banner)
 !Constant PUNYINFORM_VERSION_SUFFIX = "dev"; ! Comment out if none
 
 #Ifndef VN_1636;
@@ -11,70 +12,70 @@ Message fatalerror "*** The PunyInform library needs Inform v6.36 or later to wo
 #Endif; ! VN_
 
 #IfnDef CUSTOM_ABBREVIATIONS;
-Abbreviate "are referring to.";
-Abbreviate " fixed in place.";
+Abbreviate "Are you sure you want to ";
+Abbreviate " descriptions o";
 Abbreviate "n't understand";
 Abbreviate "(first taking ";
-Abbreviate " doesn't seem";
-Abbreviate " seems to be";
-Abbreviate "But you are";
+Abbreviate "le objects w";
+Abbreviate " pitch dark ";
+Abbreviate "There is no";
 Abbreviate "unexpected.";
-Abbreviate "description";
+Abbreviate "But you are";
+Abbreviate "use multip";
 Abbreviate "You can't ";
-Abbreviate " nothing ";
 Abbreviate "something";
 Abbreviate " carrying";
-Abbreviate "There is";
-Abbreviate "multiple";
-Abbreviate "LookMode";
-Abbreviate " already";
+Abbreviate " nothing ";
+Abbreviate " already ";
+Abbreviate "holding ";
+Abbreviate "yourself";
+Abbreviate "wearing ";
+Abbreviate "[Comment";
+Abbreviate " itself.";
 Abbreviate "irection";
-Abbreviate " (which ";
-Abbreviate "contain";
-Abbreviate "/when_o";
 Abbreviate "You are";
-Abbreviate "before";
-Abbreviate "_array";
-Abbreviate "essage";
-Abbreviate "number";
-Abbreviate "would ";
-Abbreviate "closed";
+Abbreviate " number";
+Abbreviate "Inform ";
+Abbreviate " score";
 Abbreviate "You're";
-Abbreviate "object";
-Abbreviate "That's";
-Abbreviate " the ";
-Abbreviate " you ";
-Abbreviate "which";
-Abbreviate "thing";
-Abbreviate "witch";
+Abbreviate " refer";
+Abbreviate "eed to";
+Abbreviate "which ";
+Abbreviate "switch";
+Abbreviate "achiev";
+Abbreviate "would ";
+Abbreviate " game";
+Abbreviate " not ";
+Abbreviate "close";
 Abbreviate " that";
-Abbreviate "parse";
-Abbreviate "react";
-Abbreviate "able";
-Abbreviate "can ";
-Abbreviate "name";
-Abbreviate "n't ";
-Abbreviate " to ";
-Abbreviate "_to/";
-Abbreviate "see ";
-Abbreviate "ing ";
+Abbreviate " you ";
+Abbreviate "thing";
+Abbreviate " the ";
+Abbreviate "again";
 Abbreviate "here";
+Abbreviate "ing ";
+Abbreviate " to ";
+Abbreviate "n't ";
 Abbreviate "You ";
 Abbreviate "have";
-Abbreviate "self";
-Abbreviate "ed.";
-Abbreviate " no";
-Abbreviate "est";
-Abbreviate "ut ";
+Abbreviate " ***";
+Abbreviate "can ";
 Abbreviate " in";
-Abbreviate "ope";
-Abbreviate "ent";
-Abbreviate "eve";
-Abbreviate "Th";
-Abbreviate "I ";
-Abbreviate ", ";
-Abbreviate "In";
+Abbreviate " an";
+Abbreviate " on";
+Abbreviate "pen";
+Abbreviate "thi";
+Abbreviate "ver";
+Abbreviate "ly ";
+Abbreviate "ut ";
+Abbreviate "ed.";
+Abbreviate "s.";
 Abbreviate "e.";
+Abbreviate "I ";
+Abbreviate ".^";
+Abbreviate "Th";
+Abbreviate "]^";
+Abbreviate "'s";
 #EndIf;
 
 #IfDef STATUSLINE_TIME;
@@ -138,7 +139,7 @@ Constant EXCEPT_WORD2 = 'except';
 Constant AND_WORD     = 'and';
 Constant THEN1__WD    = 'then';
 Constant comma_word   = 'comma,';  ! An "untypeable word" used to substitute
-                                   ! for commas in parse buffers
+								   ! for commas in parse buffers
 
 ! Isolating words and punctuation for reuse and potential translation
 Constant SOMETHING_STR = "something";
@@ -147,9 +148,7 @@ Constant SOMEDIRECTION_STR = "(some direction)";
 Constant IS_STR = "is ";
 Constant ARE_STR = "are ";
 
-#Ifndef DEFAULT_CAPACITY;
-Constant DEFAULT_CAPACITY = 100;
-#Endif;
+Default DEFAULT_CAPACITY = 100;
 
 #Ifdef OPTIONAL_PROVIDE_UNDO;
 #IfV3;
@@ -210,7 +209,7 @@ Property additive life   $ffff;
 Property initial;
 Property description;
 Property cant_go;
-Property found_in;         !  For fiddly reasons this can't alias
+Property found_in;		 !  For fiddly reasons this can't alias
 Property time_left;
 Property additive time_out $ffff;
 Property short_name 0;
@@ -323,61 +322,61 @@ Constant DIRECTION_COUNT = 8;
 
 #IfDef OPTIONAL_ALLOW_WRITTEN_NUMBERS;
 Array LanguageNumbers static table
-    'one' 1 'two' 2 'three' 3 'four' 4 'five' 5
-    'six' 6 'seven' 7 'eight' 8 'nine' 9 'ten' 10
-    'eleven' 11 'twelve' 12 'thirteen' 13 'fourteen' 14 'fifteen' 15
-    'sixteen' 16 'seventeen' 17 'eighteen' 18 'nineteen' 19 'twenty' 20;
+	'one' 1 'two' 2 'three' 3 'four' 4 'five' 5
+	'six' 6 'seven' 7 'eight' 8 'nine' 9 'ten' 10
+	'eleven' 11 'twelve' 12 'thirteen' 13 'fourteen' 14 'fifteen' 15
+	'sixteen' 16 'seventeen' 17 'eighteen' 18 'nineteen' 19 'twenty' 20;
 #EndIf;
 
 #Ifdef OPTIONAL_LANGUAGE_NUMBER;
 #Ifdef OPTIONAL_ALLOW_WRITTEN_NUMBERS;
 #IfV3;
 Array LanguageNumberStrings static -->
-    "thirteen"
-    "fourteen"
-    "fifteen"
-    "sixteen"
-    "seventeen"
-    "eighteen"
-    "nineteen";
+	"thirteen"
+	"fourteen"
+	"fifteen"
+	"sixteen"
+	"seventeen"
+	"eighteen"
+	"nineteen";
 #Endif;
 #Ifnot; ! Not OPTIONAL_ALLOW_WRITTEN_NUMBERS
 Array LanguageNumberStrings static -->
 	"one"
 	"two"
 	"three"
-    "four"
-    "five"
-    "six"
-    "seven"
-    "eight"
-    "nine"
-    "ten"
-    "eleven"
-    "twelve"
-    "thirteen"
-    "fourteen"
-    "fifteen"
-    "sixteen"
-    "seventeen"
-    "eighteen"
-    "nineteen";
+	"four"
+	"five"
+	"six"
+	"seven"
+	"eight"
+	"nine"
+	"ten"
+	"eleven"
+	"twelve"
+	"thirteen"
+	"fourteen"
+	"fifteen"
+	"sixteen"
+	"seventeen"
+	"eighteen"
+	"nineteen";
 #Endif; ! Not OPTIONAL_ALLOW_WRITTEN_NUMBERS
 
 Array LanguageNumberTensStrings static -->
 	"twenty"
-    "thirty"
-    "forty"
-    "fifty"
-    "sixty"
-    "seventy"
-    "eighty"
-    "ninety";
+	"thirty"
+	"forty"
+	"fifty"
+	"sixty"
+	"seventy"
+	"eighty"
+	"ninety";
 #Endif; ! OPTIONAL_LANGUAGE_NUMBER
 
 
 Constant TT_OBJECT           = 1;    ! one or more words referring to an object
-                                     ! it is one of NOUN_TOKEN etc. below
+									 ! it is one of NOUN_OBJECT etc. below
 Constant TT_PREPOSITION      = 2;    ! e.g. 'into'
 Constant TT_ROUTINE_FILTER   = 3;    ! e.g. noun=CagedCreature
 Constant TT_ATTR_FILTER      = 4;    ! e.g. edible
@@ -400,6 +399,13 @@ Constant GPR_FAIL           = -1;   ! Return values from General Parsing
 Constant GPR_PREPOSITION    = 0;    ! Routines
 Constant GPR_NUMBER         = 1;
 Constant GPR_MULTIPLE       = 2;
+Constant GPR_NOUN           = -256; ! Reparse, but as NOUN_OBJECT this time
+Constant GPR_HELD           = GPR_NOUN + 1; ! ditto for HELD_OBJECT
+Constant GPR_MULTI          = GPR_NOUN + 2; ! and so on...
+Constant GPR_MULTIHELD      = GPR_NOUN + 3;
+Constant GPR_MULTIEXCEPT    = GPR_NOUN + 4;
+Constant GPR_MULTIINSIDE    = GPR_NOUN + 5;
+Constant GPR_CREATURE       = GPR_NOUN + 6;
 Constant GPR_REPARSE        = 10000;
 
 Constant TOKEN_SINGLE_PREP   = $42; ! 66
@@ -443,6 +449,12 @@ Global location = INITIAL_LOCATION_VALUE;		! Must be the first global to show lo
 	#EndIf;
 #EndIf;
 
+Constant PARSING_REASON       = 0;  ! Possible reasons for searching scope
+Constant EACH_TURN_REASON     = 2;
+Constant REACT_BEFORE_REASON  = 3;
+Constant REACT_AFTER_REASON   = 4;
+Constant LOOPOVERSCOPE_REASON = 5;
+Constant TESTSCOPE_REASON     = 6;
 
 #Ifdef NO_SCORE;
 #Ifndef OPTIONAL_SL_NO_SCORE;
@@ -473,8 +485,8 @@ Global actor;
 Global wn;               ! word number within parse array
 Global pattern_pointer;  ! token within current pattern
 Global num_words;        ! number of words typed
-Global action;           ! the current action
-Global action_to_be;     ! the action being considered
+Global action = NULL;    ! the current action
+Global action_to_be = NULL; ! the action being considered
 Global action_reverse;   ! if parameters are in reversed order
 Global meta;             ! if the verb has the meta attribute or not
 Global update_moved;     ! if _NoteObjectAcquisitions should update moved
@@ -506,6 +518,7 @@ Global parser_one; ! Used to pass extra information between user code and parser
 Global parser_two; ! Used to pass extra information between user code and parser
 Global usual_grammar_after; ! needed for grammar property parsing
 Global deadflag;
+Global scope_reason = PARSING_REASON; ! The reason scope is being calculated, see constants *_REASON
 Global scope_modified;   ! true if daemons, each_turn etc has invalidated scope
 Global scope_objects;
 Global scope_copy_objects;
@@ -544,16 +557,31 @@ Global debug_flag = 0;
 Global timer1 = 0;
 Global timer2 = 0;
 #EndIf;
+
+Constant CLR_CURRENT         = 0;
+Constant CLR_DEFAULT         = 1;
+Constant CLR_BLACK           = 2;
+Constant CLR_RED             = 3;
+Constant CLR_GREEN           = 4;
+Constant CLR_YELLOW          = 5;
+Constant CLR_BLUE            = 6;
+Constant CLR_MAGENTA         = 7;
+Constant CLR_CYAN            = 8;
+Constant CLR_WHITE           = 9;
+
 #IfV5;
+Constant WIN_ALL     0;
+Constant WIN_STATUS  1;
+Constant WIN_MAIN    2;
 Global screen_width;
 Global statusline_current_height = 0;
 Global statusline_height     = 1;
 Global statuswin_current     = false;
 Global clr_on                = false;
-Global clr_bg                = 2;
-Global clr_fg                = 8;
-Global clr_bgstatus          = 2;
-Global clr_fgstatus          = 8;
+Global clr_bg                = CLR_BLACK;
+Global clr_fg                = CLR_WHITE;
+Global clr_fgstatus          = CLR_CURRENT;
+Global clr_fginput           = CLR_CURRENT;
 #endif;
 
 Global normal_directions_enabled = true;
@@ -576,18 +604,14 @@ Array scope-->MAX_SCOPE; ! objects visible from the current POV
 Array scope_copy-->MAX_SCOPE; ! Used to hold a copy of a scope list, for iteration
 
 Constant WORD_HIGHBIT = $8000;
-#Ifndef MAX_TIMERS;
-Constant MAX_TIMERS  32;            ! Max number timers/daemons active at once
-#Endif; ! MAX_TIMERS
+Default MAX_TIMERS  32;            ! Max number timers/daemons active at once
 Array  the_timers --> MAX_TIMERS;
 Global active_timers;               ! Number of timers/daemons active
 Global current_timer;               ! Index of the timer which is currently being executed
 
 Global PrintMsg = _PrintMsg;        ! Using a global for this saves one byte per call
 
-#Ifndef MAX_FLOATING_OBJECTS;
-Constant MAX_FLOATING_OBJECTS  32;            ! Max number of objects that have found_in property
-#Endif; ! MAX_FLOATING_OBJECTS
+Default MAX_FLOATING_OBJECTS  32;            ! Max number of objects that have found_in property
 Array floating_objects --> MAX_FLOATING_OBJECTS + 1;
 
 Global c_style = 0;
@@ -605,9 +629,13 @@ Global listing_together; ! The first object# in the group when listing together
 Global lt_value;
 #Endif;
 
+Global input_action;
+Global input_noun;
+Global input_second;
+Global input_direction;
 
-Array which_object-->MAX_WHICH_OBJECTS;       ! options for "which book?"
-Array multiple_objects-->MAX_MULTIPLE_OBJECTS;! holds nouns when multi* used
+Array which_object-->MAX_WHICH_OBJECTS+1;       ! options for "which book?"
+Array multiple_objects-->MAX_MULTIPLE_OBJECTS+1;! holds nouns when multi* used
 
 Array buffer->(MAX_INPUT_CHARS + 3);
 Array parse->(2 + 4 * (MAX_INPUT_WORDS + 1)); ! + 1 to make room for an extra word which is set to 0
@@ -642,7 +670,7 @@ Global places_score;
 #IfDef OPTIONAL_FULL_SCORE;
 #IfDef TASKS_PROVIDED;
 Default NUMBER_TASKS        1;
-Array  task_done -> NUMBER_TASKS;
+Array  task_done -> NUMBER_TASKS; ! The compiler sets all elements to 0
 #Ifndef task_scores;
 Array task_scores -> 1;
 #Endif;
@@ -673,7 +701,7 @@ Object Directions
 #Ifndef OPTIONAL_SHIP_DIRECTIONS;
 				if(_w == 'floor' or 'ground') {
 					selected_direction_index = DIRECTION_COUNT - 2;
-	        		jump _matched_and_have_set_dir_index;
+					jump _matched_and_have_set_dir_index;
 				}
 #Endif;
 			}
@@ -685,7 +713,7 @@ Object Directions
 			if((normal_directions_enabled || ship_directions_enabled) &&
 					_w == 'floor' or 'ground') {
 				selected_direction_index = DIRECTION_COUNT - 2;
-        		jump _matched_and_have_set_dir_index;
+				jump _matched_and_have_set_dir_index;
 			}
 #EndIf;
 			! no match
