@@ -95,6 +95,9 @@ Constant MSG_REMOVE_DEFAULT "Removed.";
 #Ifndef MSG_SEARCH_NOTHING_SPECIAL;
 Constant MSG_SEARCH_NOTHING_SPECIAL "You find nothing special.";
 #EndIf;
+#Ifndef MSG_OOPS_DEFAULT;
+Constant MSG_OOPS_DEFAULT "Think nothing of it.";
+#EndIf;
 #Ifndef MSG_PARSER_ONLY_TO_ANIMATE;
 Constant MSG_PARSER_ONLY_TO_ANIMATE "You can only do that to something animate.";
 #EndIf;
@@ -137,6 +140,14 @@ Constant MSG_PARSER_NOTHING_TO_AGAIN "You can hardly repeat that.";
 #Ifndef MSG_PARSER_BE_MORE_SPECIFIC;
 Constant MSG_PARSER_BE_MORE_SPECIFIC "You need to be more specific.";
 #EndIf;
+#Ifndef MSG_PARSER_NO_MULTIPLES_FOR_NPC;
+Constant MSG_PARSER_NO_MULTIPLES_FOR_NPC "You can only refer to single objects when talking to others.";
+#EndIf;
+#IfnDef OPTIONAL_NO_DARKNESS;
+#IfnDef MSG_NOW_DARK;
+Constant MSG_NOW_DARK "^It is now pitch dark in here.";
+#EndIf;
+#EndIf;
 
 #IfDef OPTIONAL_FULL_SCORE;
 #IfDef OPTIONAL_SCORED;
@@ -304,6 +315,11 @@ Default MSG_EAT_INEDIBLE = 139;
 Default MSG_OPEN_ALREADY = 140;
 Default MSG_OPEN_LOCKED = 141;
 Default MSG_PUTON_NOT_SUPPORTER = 142;
+Default MSG_PARSER_NO_IT = 143;
+Default MSG_PARSER_CANT_SEE_IT = 144;
+Default MSG_NOTIFY_ON = 145;
+Default MSG_NOTIFY_OFF = 146;
+Default MSG_ENTER_HELD 147;
 
 #IfDef OPTIONAL_PROVIDE_UNDO_FINAL;
 #Ifndef MSG_UNDO_NOTHING_DONE;
@@ -320,6 +336,16 @@ Constant MSG_UNDO_DONE "Previous turn undone.";
 #EndIf;
 #EndIf;
 
+#Ifndef MSG_COMMENT_TRANSCRIPT;
+Constant MSG_COMMENT_TRANSCRIPT "[Comment recorded]^";
+#EndIf;
+#Ifndef MSG_COMMENT_NO_TRANSCRIPT;
+#ifdef OPTIONAL_EXTENDED_METAVERBS;
+Constant MSG_COMMENT_NO_TRANSCRIPT "[Comment not recorded. To record comments, start a transcript.]^";
+#IfNot;
+Constant MSG_COMMENT_NO_TRANSCRIPT "[Comment not recorded. This game does not support transcripts.]^";
+#EndIf;
+#EndIf;
 
 #IfDef OPTIONAL_EXTENDED_VERBSET;
 #Ifndef MSG_BURN_DEFAULT;
@@ -332,7 +358,7 @@ Constant MSG_BUY_DEFAULT "Nothing is on sale.";
 Constant MSG_EMPTY_WOULDNT_ACHIEVE "That would scarcely empty anything.";
 #EndIf;
 #Ifndef MSG_RHETORICAL_QUESTION;
-Constant	MSG_RHETORICAL_QUESTION "That was a rhetorical question.";
+Constant MSG_RHETORICAL_QUESTION "That was a rhetorical question.";
 #EndIf;
 #Ifndef MSG_PRAY_DEFAULT;
 Constant MSG_PRAY_DEFAULT "Nothing practical results from your prayer.";
@@ -389,6 +415,7 @@ Default MSG_WAVE_DEFAULT 201;
 Default MSG_EMPTY_ALREADY_EMPTY 202;
 Default MSG_SET_DEFAULT 203;
 Default MSG_SET_TO_DEFAULT 204;
+Default MSG_EMPTY_NOT_CONTAINER 205;
 #EndIf;
 
 
@@ -621,6 +648,23 @@ Constant SKIP_MSG_LOOKMODE;
 #Endif;
 #Endif;
 
+#Iffalse MSG_INSERT_NOT_CONTAINER < 1000;
+#Ifndef MSG_EMPTY_NOT_CONTAINER;
+Constant SKIP_MSG_INSERT_NOT_CONTAINER;
+#Ifnot;
+#Iffalse MSG_EMPTY_NOT_CONTAINER < 1000;
+Constant SKIP_MSG_INSERT_NOT_CONTAINER;
+#Endif;
+#Endif;
+#Endif;
+
+#Ifndef OPTIONAL_NO_SCORE;
+#Iffalse MSG_NOTIFY_ON < 1000;
+#Iffalse MSG_NOTIFY_OFF < 1000;
+Constant SKIP_MSG_NOTIFY_ON;
+#Endif;
+#Endif;
+#Endif;
 
 #Ifndef OPTIONAL_NO_DARKNESS;
 #Iffalse MSG_EXAMINE_DARK < 1000;
@@ -829,34 +873,35 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #Iftrue MSG_AUTO_TAKE < 1000;
 	MSG_AUTO_TAKE:
 	! p_arg_1 = the object the player automatically picks up
-	print "(first taking ", (the) p_arg_1, ")^";
+		"(first taking ", (the) p_arg_1, ")";
 #Endif;
 #Iftrue MSG_AUTO_DISROBE < 1000;
 	MSG_AUTO_DISROBE:
 	! p_arg_1 = the object the player automatically takes off.
-		print "(first taking off ", (the) p_arg_1, ")^";
+		"(first taking off ", (the) p_arg_1, ")";
 #Endif;
 #Iftrue MSG_AUTO_DISROBE_WORN < 1000;
 	MSG_AUTO_DISROBE_WORN:
 	! p_arg_1 = the object the player would need to take off.
-		print "But you would need to take off ", (the) p_arg_1, " first.^";
+		"But you would need to take off ", (the) p_arg_1, " first.";
 #Endif;
 #IfTrue MSG_PARSER_NOTHING_TO_VERB < 1000;
 	MSG_PARSER_NOTHING_TO_VERB:
 	! p_arg_1 = the last word in player input + 1.
-		if(action == ##Drop or ##Insert && (parse + 2 + (p_arg_1 - 2) *4)-->0 == ALL_WORD) {
-			"You are not carrying anything.";
-		}  else {
-			print "There are no things available that match ~";
-			_PrintPartialMatch(verb_wordnum, p_arg_1 - 1);
-			"~.";
+		if(action == ##Drop or ##Insert) {
+			if((parse + 2 + (p_arg_1 - 2) *4) --> 0 == ALL_WORD)
+				"You are not carrying anything.";
+			if(TryNumber(verb_wordnum + 1) > 0) "You're not holding any.";
 		}
+		print "There are no things available that match ~";
+		_PrintPartialMatch(verb_wordnum, p_arg_1 - 1);
+		"~.";
 #EndIf;
 #Ifndef SKIP_MSG_PARSER_NOT_HOLDING;
 	MSG_PARSER_NOT_HOLDING, MSG_AUTO_TAKE_NOT_HELD, MSG_WAVE_NOTHOLDING:
 	! p_arg_1 = the object which the player must be holding to perform the
 	! action but isn't.
-		print_ret "But you are not holding ", (the) p_arg_1, ".";
+		"But you are not holding ", (the) p_arg_1, ".";
 #Endif;
 #IfTrue MSG_PARSER_PARTIAL_MATCH < 1000;
 	MSG_PARSER_PARTIAL_MATCH:
@@ -868,19 +913,19 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #IfTrue MSG_PARSER_CANT_TALK < 1000;
 	MSG_PARSER_CANT_TALK:
 	! p_arg_1 = the object which can't be talked to.
-		print_ret "You can't talk to ", (the) p_arg_1, ".";
+		"You can't talk to ", (the) p_arg_1, ".";
 #EndIf;
 #IfTrue MSG_PARSER_NO_NEED_REFER_TO < 1000;
 	MSG_PARSER_NO_NEED_REFER_TO:
 		print "You don't need to refer to ~";
 		_PrintUnknownWord();
-		print_ret "~ in this game.";
+		"~ in this game.";
 #EndIf;
 #IfTrue MSG_PARSER_DONT_UNDERSTAND_WORD < 1000;
 	MSG_PARSER_DONT_UNDERSTAND_WORD:
-			print "Sorry, I don't understand what ~";
-			_PrintUnknownWord();
-			print_ret "~ means.";
+		print "Sorry, I don't understand what ~";
+		_PrintUnknownWord();
+		"~ means.";
 #EndIf;
 #IfTrue MSG_PARSER_BAD_PATTERN_PREFIX < 1000;
 	MSG_PARSER_BAD_PATTERN_PREFIX:
@@ -891,6 +936,14 @@ Constant SKIP_MSG_EXAMINE_DARK;
 	MSG_PARSER_BAD_PATTERN_SUFFIX:
 		"~. Please try again.";
 #EndIf;
+#IfTrue MSG_PARSER_NO_IT < 1000;
+	MSG_PARSER_NO_IT:
+		"I don't know what ~",(address) p_arg_1, "~ refers to.";
+#EndIf;
+#IfTrue MSG_PARSER_CANT_SEE_IT < 1000;
+	MSG_PARSER_CANT_SEE_IT:
+		"You can't see ~",(address) p_arg_1, "~ (", (name) p_arg_2, ") at the moment.";
+#EndIf;
 #Ifndef SKIP_MSG_CLOSE_NOT_OPEN;
 	MSG_CLOSE_NOT_OPEN, MSG_TOUCHABLE_FOUND_CLOSED,
 	MSG_PARSER_CONTAINER_ISNT_OPEN:
@@ -921,7 +974,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #IfTrue MSG_EXAMINE_CLOSED < 1000;
 	MSG_EXAMINE_CLOSED:
 	! p_arg_1 = the examines object (which is closed).
-		print_ret (The) p_arg_1, " ", (isorare) p_arg_1, " closed.";
+		print_ret (CObjIs) p_arg_1, " closed.";
 #Endif;
 #IfTrue MSG_REMOVE_NOT_HERE < 1000;
 	MSG_REMOVE_NOT_HERE:
@@ -929,7 +982,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #EndIf;
 #IfTrue MSG_SEARCH_IN_IT_ISARE < 1000;
 	MSG_SEARCH_IN_IT_ISARE:
-		print (The) noun, " contains ";
+		print (The) noun, " contain", (SingularS) noun, " ";
 		PrintContents(0, noun);
 		".";
 #EndIf;
@@ -953,7 +1006,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 #EndIf;
 #IfTrue MSG_EXAMINE_ONOFF < 1000;
 	MSG_EXAMINE_ONOFF:
-		print_ret (The) noun, " ", (IsOrAre) noun, " currently switched ", (onoff) noun, ".";
+		print_ret (CObjIs) noun, " currently switched ", (onoff) noun, ".";
 #EndIf;
 #IfTrue MSG_EAT_DEFAULT < 1000;
 	MSG_EAT_DEFAULT:
@@ -970,12 +1023,12 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 #IfTrue MSG_TAKE_BELONGS < 1000;
 	MSG_TAKE_BELONGS:
 		! p_arg_1 = the object that is held by p_arg_2
-		print_ret (The) p_arg_1, " seems to belong to ", (the) p_arg_2, ".";
+		print_ret (The) p_arg_1, " seem", (SingularS) p_arg_1, " to belong to ", (the) p_arg_2, ".";
 #EndIf;
 #IfTrue MSG_TAKE_PART_OF < 1000;
 	MSG_TAKE_PART_OF:
 		! p_arg_1 = the object that is part of p_arg_2
-		print_ret (The) p_arg_1, " seems to be part of ", (the) p_arg_2, ".";
+		print_ret (The) p_arg_1, " seem", (SingularS) p_arg_1, " to be part of ", (the) p_arg_2, ".";
 #EndIf;
 #Ifndef OPTIONAL_NO_DARKNESS;
 #Ifndef SKIP_MSG_EXAMINE_DARK;
@@ -1043,7 +1096,7 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 #IfDef OPTIONAL_FULL_SCORE;
 		print ", give the FULL score for that game";
 #EndIf;
-		if(deadflag == 2 && AMUSING_PROVIDED == 0) print ", see some suggestions for AMUSING things to do";
+		if(AMUSING_PROVIDED == 0 && deadflag == 2) print ", see some suggestions for AMUSING things to do";
 		print " or QUIT? ";
 		rtrue;
 #EndIf;
@@ -1071,20 +1124,37 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 			print "enter ", (the) parent(noun);
 		" first.";
 #EndIf;
-#IfTrue MSG_INSERT_NOT_CONTAINER < 1000;
+#IfTrue MSG_ENTER_HELD < 1000;
+	MSG_ENTER_HELD:
+		"You can't enter ", (the) noun, " while holding ", (ItOrThem) noun, ".";
+#EndIf;
+#Ifndef SKIP_MSG_INSERT_NOT_CONTAINER;
+#ifdef MSG_EMPTY_NOT_CONTAINER;
+	MSG_INSERT_NOT_CONTAINER, MSG_EMPTY_NOT_CONTAINER:
+#Ifnot;
 	MSG_INSERT_NOT_CONTAINER:
+#Endif;
 		! p_arg_1 = the object that can't contain things
 		print_ret (The) p_arg_1, " can't contain things.";
 #Endif;
 #IfTrue MSG_YES_OR_NO < 1000;
 	MSG_YES_OR_NO:
 		print "Please answer yes or no: ";
+		rtrue;
 #EndIf;
 #IfTrue MSG_RESTART_CONFIRM < 1000;
 	MSG_RESTART_CONFIRM:
 		print "Are you sure you want to restart? ";
+		rtrue;
 #Endif;
+
 #Ifndef NO_SCORE;
+#Ifndef SKIP_MSG_NOTIFY_ON;
+	MSG_NOTIFY_ON, MSG_NOTIFY_OFF:
+		print "Score notification o";
+		if(p_msg == MSG_NOTIFY_ON) "n.";
+		"ff.";
+#Endif;
 #Iftrue MSG_PARSER_NEW_SCORE < 1000;
 	MSG_PARSER_NEW_SCORE:
 		! p_arg_1 = the old score
@@ -1098,7 +1168,7 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 		}
 		print " by ", p_arg_2, " point";
 		if(p_arg_2 > 1) print "s";
-		print ".]^";
+		".]";
 #Endif;
 #Endif;
 
@@ -1113,7 +1183,7 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 #IfTrue MSG_EMPTY_ALREADY_EMPTY < 1000;
 	MSG_EMPTY_ALREADY_EMPTY:
 		! p_arg_1 = the object that is already empty
-		print_ret (The) p_arg_1, " ", (isorare) p_arg_1, " empty already.";
+		print_ret (CObjIs) p_arg_1, " empty already.";
 #EndIf;
 #IfTrue MSG_SET_DEFAULT < 1000;
 	MSG_SET_DEFAULT:
@@ -1135,16 +1205,16 @@ default:
 	}
 ];
 
-[ ThatorThose obj;
-	if (obj has pluralname) print "those"; else print "that";
+[ ThatorThose p_obj;
+	if (p_obj has pluralname) print "those"; else print "that";
 ];
 
-[ ItorThem obj;
-	if (obj == player) { print "yourself"; rtrue; }
-	if (obj has pluralname) { print "them"; rtrue; }
-	if (obj has animate) {
-		if (obj has female) { print "her"; rtrue; }
-		if (obj hasnt neuter) { print "him"; rtrue; }
+[ ItorThem p_obj;
+	if (p_obj == player)		{ print "yourself"; rtrue; }
+	if (p_obj has pluralname)	{ print "them"; rtrue; }
+	if (p_obj has animate) {
+		if (p_obj has female)	{ print "her"; rtrue; }
+		if (p_obj hasnt neuter)	{ print "him"; rtrue; }
 	}
 	print "it";
 ];
@@ -1153,16 +1223,16 @@ default:
 	print (The) p_obj, " ", (isorare) p_obj;
 ];
 
-[ IsorAre obj;
-	if (obj has pluralname || obj == player) print "are"; else print "is";
+[ IsorAre p_obj;
+	if (p_obj has pluralname || p_obj == player) print "are"; else print "is";
 ];
 
-[ CTheyreorThats obj;
-	if (obj == player)			 { print "You're"; return; }
-	if (obj has pluralname)		{ print "They're"; return; }
-	if (obj has animate) {
-		if (obj has female)		{ print "She's"; return; }
-		if (obj hasnt neuter) { print "He's"; return; }
+[ CTheyreorThats p_obj;
+	if (p_obj == player)		{ print "You're"; return; }
+	if (p_obj has pluralname)	{ print "They're"; return; }
+	if (p_obj has animate) {
+		if (p_obj has female)	{ print "She's"; return; }
+		if (p_obj hasnt neuter) { print "He's"; return; }
 	}
 	print "That's";
 ];
@@ -1175,13 +1245,15 @@ default:
 	CTheyreorThats(p_obj);
 ];
 
-[OnOff obj;
-	if(obj has on) print "on";
+[ OnOff p_obj;
+	if(p_obj has on) print "on";
 	else print "off";
 	return;
 ];
 
-
+[ SingularS p_obj;
+	if(p_obj hasnt pluralname && p_obj ~= player) print "s";
+];
 !
 ! Error messages
 !
@@ -1194,13 +1266,16 @@ Constant ERR_TOO_MANY_FLOATING 6;
 Constant ERR_NOT_DIR_PROP 7;
 Constant ERR_NOT_FAKE_OBJ 8;
 Constant ERR_ILLEGAL_CHOOSEOBJNO 9;
+Constant ERR_BUFFER_OVERRUN 10;
 
 [_RunTimeError p_err p_obj _parent;
-	print "[PunyInform error: ";
+	print "^[PunyInform error: ";
 	if(p_err ofclass string)
 		print (string) p_err;
 	else {
+		print p_err;
 #IfTrue RUNTIME_ERRORS == RTE_VERBOSE;
+		print " - ";
 		switch(p_err) {
 		ERR_TOO_MANY_TIMERS_DAEMONS:
 			print "Too many timers/daemons";
@@ -1211,7 +1286,7 @@ Constant ERR_ILLEGAL_CHOOSEOBJNO 9;
 		ERR_UNKNOWN_MSGNO:
 			print "Unknown message#";
 		ERR_INVALID_DIR_PROP:
-			print "Invalid direction prop in GoSub";
+			print "GoSub called with invalid direction property";
 		ERR_TOO_MANY_FLOATING:
 			print "Too many floating objects";
 		ERR_NOT_DIR_PROP:
@@ -1219,19 +1294,21 @@ Constant ERR_ILLEGAL_CHOOSEOBJNO 9;
 		ERR_NOT_FAKE_OBJ:
 			print "FakeObjToDirProp called with non-fakeobj";
 		ERR_ILLEGAL_CHOOSEOBJNO:
-			print "ChooseObjectFinal_(Pick or Discard) called with nonexistent array index.";
+			print "ChooseObjectsFinal_(Pick or Discard) called with nonexistent array index";
+#Ifdef DEBUG;
+		ERR_BUFFER_OVERRUN:
+			print "Buffer overrun: Printing too many characters to a buffer";
+#Endif;
 		default:
-			print "Unknown error (", p_err, ")";
+			print "Unknown error";
 		}
-#IfNot;
-		print p_err;
 #EndIf;
 	}
 	print "]^";
 #IfTrue RUNTIME_ERRORS == RTE_VERBOSE;
 	if(p_obj ofclass object) {
 		_parent = parent(p_obj);
-		print "Offending object: ", (the) p_obj, " (", p_obj, ") in ", (name) _parent, " (", _parent, ")^"; 
+		print "Offending object: ", (the) p_obj, " (", p_obj, ") in ", (name) _parent, " (", _parent, ")^";
 	}
 #Ifnot;
 	_parent = p_obj; ! Avoid compiler warning
