@@ -105,7 +105,9 @@ Include "grammar.h";
 	}
 ];
 
+#Ifdef OPTIONAL_NON_FLASHING_STATUSLINE;
 Array cursor_pos --> 2;
+#Endif;
 
 [ _StatusLineHeight p_height;
 	if (statusline_current_height ~= p_height) {
@@ -144,6 +146,7 @@ Array cursor_pos --> 2;
 
 Array _TenSpaces static -> "          ";
 
+#Ifdef OPTIONAL_NON_FLASHING_STATUSLINE;
 [ _PrintSpacesOrMoveBack p_col p_string _current_col;
 	p_col = screen_width - p_col;
 
@@ -158,6 +161,17 @@ Array _TenSpaces static -> "          ";
 	if(p_string)
 		print (string) p_string;
 ];
+#Ifnot;
+[ _PrintSpacesOrMoveBack p_col p_string;
+	p_col = screen_width - p_col;
+
+	_MoveCursor(1, p_col);
+
+	if(p_string)
+		print (string) p_string;
+];
+#Endif;
+
 
 Constant ONE_SPACE_STRING = " ";
 
@@ -270,6 +284,10 @@ Constant ONE_SPACE_STRING = " ";
 
 	_StatusLineHeight(statusline_height);
 	_MoveCursor(1, 1); ! This also sets the upper window as active.
+#Ifndef OPTIONAL_NON_FLASHING_STATUSLINE;
+	FastSpaces(screen_width);
+	_MoveCursor(1, 1);
+#Endif;
 	if(screen_width > 66) @print_char ' ';
 
 	_visibility_ceiling = ScopeCeiling(player);
@@ -301,7 +319,9 @@ Constant ONE_SPACE_STRING = " ";
 #Endif;
 
 	! Regardless of what kind of status line we have printed, print spaces to the end.
+#Ifdef OPTIONAL_NON_FLASHING_STATUSLINE;
 	_PrintSpacesOrMoveBack(-1);
+#Endif;
 	_MainWindow(); ! set_window
 ];
 #EndIf;
@@ -829,15 +849,17 @@ Constant ONE_SPACE_STRING = " ";
 #Ifndef OPTIONAL_NO_DARKNESS;
 	if(p_obj == thedark && p_prop ~= initial or short_name or description) p_obj = real_location;
 #Endif;
+	@push sw__var;
 	if(p_switch == 0) sw__var = action; else sw__var = p_switch;
-	if (p_prop >= INDIV_PROP_START && p_obj.&p_prop == 0) rfalse;
-#Ifdef OPTIONAL_MANUAL_SCOPE;
-	return p_obj.p_prop();
-#Ifnot;
-	p_switch = p_obj.p_prop(); ! Repurposing p_switch
-	scope_modified = true;
-	return p_switch;
+	if (p_prop < INDIV_PROP_START || p_obj.&p_prop ~= 0) {
+		p_switch = p_obj.p_prop();
+#Ifndef OPTIONAL_MANUAL_SCOPE;
+		if(p_obj.#p_prop > 2 || p_obj.p_prop ~= NULL or 0)
+			scope_modified = true;
 #Endif;
+	}
+	@pull sw__var;
+	return p_switch;
 ];
 
 [ PrintOrRun p_obj p_prop p_no_string_newline _val;
@@ -2205,21 +2227,26 @@ Object thedark "Darkness"
 	return p_fake_obj - FAKE_N_OBJ + N_TO_CONST;
 ];
 
-[ _InitObjects _i _k _stop;
-	_stop = top_object + 1;
-	for(_i = Directions : _i < _stop : _i++) {
+[ _InitObjects _i _k _v;
+	_i = Directions;
+.objloop;
 #Ifndef OPTIONAL_MANUAL_REACTIVE;
+		@get_prop_addr _i react_before -> _v;
+		@jz _v ?~is_reactive;
+		@get_prop_addr _i react_after -> _v;
+		@jz _v ?~is_reactive;
+		@get_prop_addr _i each_turn -> _v;
+		@jz _v ?~is_reactive;
+		@get_prop_addr _i add_to_scope -> _v;
+		@jz _v ?~is_reactive;
 #Ifdef OPTIONAL_REACTIVE_PARSE_NAME;
-		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
-				_i.&add_to_scope ~= 0 || _i.&parse_name ~= 0)
-			give _i reactive;
-#Ifnot;
-		if(_i.&react_before ~= 0 || _i.&react_after ~= 0 || _i.&each_turn ~= 0 ||
-				_i.&add_to_scope ~= 0)
-			give _i reactive;
+		@get_prop_addr _i parse_name -> _v;
+		@jz _v ?~is_reactive;
 #Endif;
 #Endif;
-		if(_i.&found_in) {
+.back_to_objloop;
+		@get_prop_addr _i found_in -> _v;
+		@jz _v ?not_floating;
 #IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
 			if(_k >= MAX_FLOATING_OBJECTS) {
 				_RunTimeError(ERR_TOO_MANY_FLOATING);
@@ -2227,8 +2254,15 @@ Object thedark "Darkness"
 			}
 #EndIf;
 			floating_objects-->(_k++) = _i;
-		}
-	}
+.not_floating;
+	@inc_chk _i top_object ?~objloop;
+	return;
+	
+#Ifndef OPTIONAL_MANUAL_REACTIVE;
+.is_reactive;
+	give _i reactive;
+	jump back_to_objloop;
+#EndIf;
 ];
 
 [ SetTime p_time p_step;
